import java.util.Collections;
import java.util.Properties;

import junit.framework.TestCase;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.change.ChangeDescription;
import org.eclipse.emf.ecore.change.ChangePackage;
import org.eclipse.emf.ecore.change.FeatureChange;
import org.eclipse.emf.ecore.change.impl.EObjectToChangesMapEntryImpl;
import org.eclipse.emf.ecore.change.util.ChangeRecorder;
import org.eclipse.emf.teneo.PersistenceOptions;
import org.eclipse.emf.teneo.StoreValidationException;
import org.eclipse.emf.teneo.hibernate.HbDataStore;
import org.eclipse.emf.teneo.hibernate.HbHelper;
import org.eclipse.emf.teneo.hibernate.resource.HibernateXMLResource;
import org.eclipse.example.library.Library;
import org.eclipse.example.library.LibraryFactory;
import org.eclipse.example.library.LibraryPackage;
import org.hibernate.cfg.Environment;

public class Test extends TestCase {

	public void testIt() {
		initDataStore();

		{
			final HibernateXMLResource hbResource = new HibernateXMLResource(
					URI.createURI("hb:/?dsname=library")) {
				protected void validateContents()
						throws StoreValidationException {
				}
			};
			ChangeRecorder rec = new ChangeRecorder(hbResource);
			final Library library = LibraryFactory.eINSTANCE.createLibrary();
			hbResource.getContents().add(library);
			library.setName("test");
			// final Writer writer = LibraryFactory.eINSTANCE.createWriter();
			// writer.setName("test");
			// library.getWriters().add(writer);
			ChangeDescription description = rec.endRecording();
			library.getChangesets().add(description);
			hbResource.save(Collections.EMPTY_MAP);
		}

		{
			final HibernateXMLResource hbResource = new HibernateXMLResource(
					URI.createURI("hb:/?dsname=library&query=From Library")) {
				protected void validateContents()
						throws StoreValidationException {
				}
			};
			hbResource.load(Collections.EMPTY_MAP);
			for (Object o : hbResource.getContents()) {
				if (o instanceof Library) {
					final Library l = (Library)o;
					final ChangeDescription cd = (ChangeDescription)l.getChangesets().get(0);
					System.err.println(cd.getObjectChanges().size());
					for (Object k : cd.getObjectChanges()) {
						System.err.println(k);
						for (FeatureChange fc : ((EObjectToChangesMapEntryImpl)k).getTypedValue()) {
							System.err.println(fc);
						}
					}
				}
			}
		}
	}

	private HbDataStore initDataStore() {
		// Set the database information, Environment is
		// org.hibernate.cfg.Environment
		final Properties props = new Properties();
		props.setProperty(Environment.DRIVER, "com.mysql.jdbc.Driver");
		props.setProperty(Environment.USER, "root");
		props
				.setProperty(Environment.URL,
						"jdbc:mysql://127.0.0.1:3306/markus");
		props.setProperty(Environment.PASS, "root");
		props.setProperty(Environment.DIALECT,
				org.hibernate.dialect.MySQL5InnoDBDialect.class.getName());
		props.setProperty(Environment.HBM2DDL_AUTO, "update");// "create-drop");

		props.setProperty(PersistenceOptions.EMAP_AS_TRUE_MAP, "false");

		props.setProperty(PersistenceOptions.PERSISTENCE_XML,
				"featurechange.persistence.xml");

		// Initialize create the HbDataStore
		HbDataStore hbds = HbHelper.INSTANCE.createRegisterDataStore("library");
//		hbds.setEPackages(new EPackage[] { LibraryPackage.eINSTANCE,
//				EcorePackage.eINSTANCE, ChangePackage.eINSTANCE });
		hbds.setEPackages(new EPackage[] { LibraryPackage.eINSTANCE, ChangePackage.eINSTANCE });
		hbds.setProperties(props);
		try {
			hbds.initialize();
			System.err.println(hbds.getMappingXML());
			return hbds;
		} catch (Exception e) {
			e.printStackTrace(System.err);
			throw new Error(e);
		}
	}

}
